"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.importFixName = void 0;
const codefix_provider_1 = require("../codefix_provider");
const hash_meta_1 = require("../module_resolver/hash_meta");
exports.importFixName = "import";
// const importFixId = "fixMissingImport";
const errorCodes = [
    2304,
    2552,
    2663,
    2662,
    2503,
    2686,
    2693,
];
function replaceCodeActions(codeFixActions) {
    for (const codeAction of codeFixActions) {
        if (codeAction.fixName !== exports.importFixName) {
            continue;
        }
        const matchs = codeAction.description.match(/\.\..+deno\/deps\/https?\/.+\/\w{64}/);
        if (matchs == null || matchs.length === 0) {
            continue;
        }
        const originImport = matchs[0];
        const meta = hash_meta_1.HashMeta.create(`${originImport}.metadata.json`);
        if (meta == null) {
            continue;
        }
        const newImport = meta.url.href;
        codeAction.description = codeAction.description.replace(originImport, newImport);
        for (const change of codeAction.changes) {
            for (const textChange of change.textChanges) {
                textChange.newText = textChange.newText.replace(originImport, newImport);
            }
        }
    }
}
codefix_provider_1.registerCodeFix({
    errorCodes,
    replaceCodeActions,
});
//# sourceMappingURL=import_fixes.js.map